"""
Class for generating, storing and rendering levels
"""
import Numeric
import pygame
import tile
import os
from OpenGL.GL import *
from OpenGL.GLU import *



class Level:
	def __init__(self,file,tileset):
		lfile = open(os.path.join('leveldata',file),'r').readlines()
		self.size = (len(lfile[0])-1, len(lfile))
		self.tileset = tileset
		self.levelData = [[None]*self.size[0] for i in range(self.size[1])]
		self.triggers = {}
		self.objects = []

		floorTile = tile.floor(tileset,"test")
		backWall = tile.backWall(tileset,"test")
		leftFrontWall = tile.leftFrontWall(tileset,"test")
		rightFrontWall = tile.rightFrontWall(tileset,"test")
		bothFrontWall = tile.bothFrontWall(tileset,"test")
		onlyTop = tile.onlyTop(tileset,"test")
		lelev = tile.elevator(tileset.tBox.get("elevatorleft"), tileset,"test")
		relev = tile.elevator(tileset.tBox.get("elevatorright"),tileset,"test")


		for y in range(self.size[1]):
			for x in range(self.size[0]):
				char = lfile[y][x]
				if char == ',':
					self.levelData[y][x] = floorTile 
				elif char == 'w':
					self.levelData[y][x] = backWall
				elif char == '+':
					self.levelData[y][x] = bothFrontWall
				elif char == 'r':
					self.levelData[y][x] = rightFrontWall
				elif char == 'l':
					self.levelData[y][x] = leftFrontWall
				elif char == 't':
					self.levelData[y][x] = onlyTop
				elif char == 'u':
					self.levelData[y][x] = lelev
				elif char == 'v':
					self.levelData[y][x] = relev
				else:
					if not self.triggers.has_key(char):
						self.triggers[char]=[]
					self.triggers[char].append((y,x))

	def render(self):
		# Copy list
		nlist = list(self.objects)
		# Render floor first
		for y in range(self.size[0]-1,-1,-1):
			for x in range(self.size[1]):
				ttile = self.levelData[x][y]
				if ttile:
					if ttile.isFloor():
						ttile.renderBack((x,y))

		for y in range(self.size[0]-1,-1,-1):
			for x in range(self.size[1]):
				olist = nlist
				ttile = self.levelData[x][y]
				if ttile:
					if not ttile.isFloor():
						ttile.renderBack((x,y))
				if olist:
					nlist = []	
					for obj in olist:
						if (int(obj.posx) == x) and (int(obj.posy) == y):
							self.renderObject(obj)
						else:
							nlist.append(obj)
				if ttile:
					glColor4d(1,1,1,0.5)	
					ttile.renderFront((x,y))
					glColor4d(1,1,1,1)	


	def renderObject(self,obj):
		# Render Objects
		glPushMatrix()
		glDisable(GL_BLEND)
		glTranslated(obj.posx,obj.posy,0)
		glRotatef(-45,0,0,-1)
		glRotatef(-60,1,0,0)
		glTranslated(-obj.posx,-obj.posy,0)
		glEnable(GL_BLEND)
		obj.render()
		glPopMatrix()



def main():
	l = Level("leveldata/testlevel")

if __name__ == '__main__':
	main()
